from mockfs import replace_builtins, restore_builtins
import pkg_resources
import pytest
from jasmine.config import Config
from mock import Mock

import jasmine.standalone

@pytest.fixture
def app():
    try:
        reload(jasmine.standalone)
    except NameError:
        import imp
        imp.reload(jasmine.standalone)

    from jasmine.standalone import app
    app.testing = True
    return app


@pytest.fixture
def template():
    return pkg_resources.resource_string('jasmine.django.templates', 'runner.html')


@pytest.fixture
def mockfs(request):
    mfs = replace_builtins()
    mfs.add_entries({
        "/spec/javascripts/support/jasmine.yml": """
        src_dir: src
        spec_dir: spec
        """
    })
    request.addfinalizer(lambda: restore_builtins())
    return mfs

@pytest.mark.usefixtures('mockfs')
def test__favicon(monkeypatch, app):
    monkeypatch.setattr(pkg_resources, 'resource_stream', lambda package, filename: [])

    with app.test_client() as client:
        rv = client.get("/jasmine_favicon.png")

        assert rv.status_code == 200

@pytest.mark.usefixtures('mockfs')
def test__before_first_request(monkeypatch, app):
    monkeypatch.setattr(pkg_resources, 'resource_stream', lambda package, filename: [])

    assert not hasattr(app, 'jasmine_config')

    with app.test_client() as client:
        rv = client.get("/jasmine_favicon.png")
        assert rv.status_code == 200

    assert app.jasmine_config is not None


def test__serve(mockfs, app):
    mockfs.add_entries({
        "/src/main.css": "CSS",
        "/src/main.js": "JS",
        "/src/main.png": "PNG"
    })

    with app.test_client() as client:
        rv = client.get("/__src__/main.css")

        assert rv.status_code == 200
        assert rv.headers['Content-Type'] == 'text/css; charset=utf-8'

        rv = client.get("/__src__/main.js")

        assert rv.status_code == 200
        assert rv.headers['Content-Type'] == 'application/javascript'

        rv = client.get("/__src__/main.png")

        assert rv.status_code == 200
        assert rv.headers['Content-Type'] == 'application/octet-stream'


def test__run(template, mockfs, monkeypatch, app):
    monkeypatch.setattr(pkg_resources, 'resource_listdir', lambda package, dir: ['jasmine.js', 'boot.js', 'node_boot.js'])
    monkeypatch.setattr(pkg_resources, 'resource_string', lambda package, filename: template)

    monkeypatch.setattr(Config, 'src_files', lambda self: ['main.js', 'main_spec.js'])
    monkeypatch.setattr(Config, 'stylesheets', lambda self: ['main.css'])

    with app.test_client() as client:
        client.get("/")  # call app before_first_request so that the app has a jasmine_config to mock
        app.jasmine_config.reload = Mock()

        rv = client.get("/")

        assert app.jasmine_config.reload.called
        assert rv.status_code == 200

        html = rv.get_data(as_text=True)

        assert """<script src="__jasmine__/jasmine.js" type="text/javascript"></script>""" in html
        assert """<script src="__jasmine__/boot.js" type="text/javascript"></script>""" in html
        assert """<script src="__src__/main.js" type="text/javascript"></script>""" in html
        assert """<script src="__src__/main_spec.js" type="text/javascript"></script>""" in html
        assert """<link rel="stylesheet" href="__src__/main.css" type="text/css" media="screen"/>""" in html

        assert """<script src="__jasmine__/node_boot.js" type="text/javascript"></script>""" not in html



import os
import re
import pkg_resources

from flask import Flask, render_template_string, make_response, send_file

from jasmine_core import Core
from jasmine.config import Config

# instance_path set to work around: https://bitbucket.org/hpk42/pytest/issue/317
app = Flask(__name__, instance_path=os.getcwd())
app.debug = True


@app.before_first_request
def init():
    project_path = os.path.realpath(os.path.dirname(__name__))

    config_file = os.path.join(project_path, "spec/javascripts/support/jasmine.yml")

    if not os.path.exists(config_file):
        print("Could not find your config file at {0}".format(config_file))

    app.jasmine_config = Config(config_file, project_path=project_path)

    app.filetype_mapping = {
        'src': app.jasmine_config.src_dir,
        'spec': app.jasmine_config.spec_dir
    }


@app.route("/__<filetype>__/<path:filename>")
def serve(filetype, filename):
    if filetype == 'jasmine':
        contents = pkg_resources.resource_string('jasmine_core', filename)
    else:
        path = os.path.join(os.getcwd(), app.filetype_mapping[filetype](), filename)
        contents = open(path, 'r').read()

    response = make_response(contents)
    if re.match(r'^.*\.js$', filename):
        response.mimetype = 'application/javascript'
    elif re.match(r'^.*\.css$', filename):
        response.mimetype = 'text/css'
    else:
        response.mimetype = 'application/octet-stream'

    return response


@app.route("/")
def run():
    app.jasmine_config.reload()
    context = {
        'css_files': app.jasmine_config.stylesheet_urls(),
        'js_files': app.jasmine_config.script_urls()
    }

    template = pkg_resources.resource_string('jasmine.django.templates', 'runner.html')

    return render_template_string(template.decode(), **context)

@app.route('/jasmine_favicon.png')
def favicon():
    return send_file(pkg_resources.resource_stream('jasmine_core.images', 'jasmine_favicon.png'), mimetype='image/png')



import pytest
from mock import Mock


@pytest.fixture
def config():
    from jasmine.config import Config

    mock_config = Mock(spec=Config, autospec=True)
    mock_config.script_urls.return_value = ["__jasmine__/jasmine.js", "__boot__/boot.js"]
    mock_config.stylesheet_urls.return_value = ["__jasmine__/jasmine.css", "__src__/css/user.css"]

    return mock_config


@pytest.fixture
def response(rf, config):
    from jasmine.django.views import JasmineRunner

    request = rf.get("")

    return JasmineRunner.as_view(template_name="runner.html", config=config)(request)


def test_js_files(config, response):
    assert response.context_data['js_files'] == config.script_urls()
    assert response.context_data['css_files'] == config.stylesheet_urls()


def test_reload_on_each_request(config, response):
    config.reload.assert_called_once_with()


from yaml import load

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from .utils import iglob
import os

from jasmine_core import Core


class Config(object):
    def __init__(self, yaml_file, project_path=os.getcwd()):
        self.yaml_file = yaml_file
        self.project_path = project_path
        self._load()

    def _uniq(self, items, idfun=None):
        # order preserving

        if idfun is None:
            def idfun(x): return x
        seen = {}
        result = []
        for item in items:
            marker = idfun(item)
            # in old Python versions:
            # if seen.has_key(marker)
            # but in new ones:
            if marker in seen:
                continue

            seen[marker] = 1
            result.append(item)
        return result

    def _glob_filelist(self, filelist, relative_to, default=[]):
        paths = self.yaml.get(filelist) or default

        paths = [self._make_absolute(path, relative_to) for path in paths]
        paths = [self._expland_globs(path) for path in paths]
        paths = sum(paths, [])
        relpaths = [self._make_relative(path, relative_to) for path in paths]

        #fix py26 relpath from root bug http://bugs.python.org/issue5117
        return [relpath[3:] if relpath.startswith("../") else relpath for relpath in relpaths]

    def _make_absolute(self, path, relative_to):
        return (path if path.startswith('http')
        else 
          os.path.abspath(os.path.join(self.project_path, relative_to, path)) )

    def _expland_globs(self, path):
        return ([path] if path.startswith('http')
        else 
          self._glob_paths([path]))

    def _make_relative(self, path, relative_to):
        return (path if path.startswith('http')
        else os.path.relpath(path, relative_to) )

    def _glob_paths(self, paths):
        files = []

        for src_glob in paths:
            files.extend([os.path.abspath(x) for x in iglob(src_glob)])

        return list(self._uniq(files))

    def _extract_urls(self, filelist):
      local_files = [x for x in filelist if 'http' not in x]
      urls = [x for x in filelist if 'http' in x]
      return local_files, urls

    def _load(self):
        with open(self.yaml_file, 'rU') as f:
            self.yaml = load(f, Loader=Loader) or {}

    def reload(self):
        self._load()

    def src_files(self):
        return self._glob_filelist('src_files', self.src_dir())

    def stylesheets(self):
        return self._glob_filelist('stylesheets', self.src_dir())

    def helpers(self):
        default_helpers = os.path.join(self.project_path, self.spec_dir(), 'helpers/**/*.js')

        return self._glob_filelist('helpers', self.spec_dir(), default=[default_helpers])

    def spec_files(self):
        default_spec = os.path.join(self.project_path, self.spec_dir(), "**/*[sS]pec.js")

        return self._glob_filelist('spec_files', self.spec_dir(), default=[default_spec])

    def src_dir(self):
        return self.yaml.get("src_dir") or self.project_path

    def spec_dir(self):
        return self.yaml.get("spec_dir") or 'spec/javascripts'

    def _prefix_src_underscored(self, path):
      return (path if path.startswith('http')
          else "__src__/{0}".format(path)
      )

    def script_urls(self):
        core_js_files = Core.js_files()
        if 'node_boot.js' in core_js_files:
            core_js_files.remove('node_boot.js')
        return \
            ["__jasmine__/{0}".format(core_js) for core_js in core_js_files] + \
            [self._prefix_src_underscored(src_file) for src_file in self.src_files()] + \
            ["__spec__/{0}".format(helper) for helper in self.helpers()] + \
            ["__spec__/{0}".format(spec_file) for spec_file in self.spec_files()]

    def stylesheet_urls(self):
        return \
            ["__jasmine__/{0}".format(core_css) for core_css in Core.css_files()] + \
            ["__src__/{0}".format(css_file) for css_file in self.stylesheets()]



from six.moves import builtins

import os
import errno
import sys

from mockfs import replace_builtins, restore_builtins
from mock import MagicMock
import pytest

from yaml import load

from jasmine.entry_points import continuous_integration, _query, standalone, mkdir_p, install
from jasmine.ci import CIRunner
from jasmine.standalone import app as App


@pytest.fixture
def mockfs(request):
    mfs = replace_builtins()
    request.addfinalizer(lambda: restore_builtins())
    return mfs


@pytest.fixture
def mockfs_with_config(request):
    mfs = replace_builtins()
    mfs.add_entries({
        "/spec/javascripts/support/jasmine.yml": """
        src_dir: src
        spec_dir: spec
        """
    })
    request.addfinalizer(lambda: restore_builtins())
    return mfs


@pytest.fixture
def mock_CI_run(request):
    CIRunner.run = MagicMock(name='run')

def test_ci_config_check(mockfs, monkeypatch, mock_CI_run):
    monkeypatch.setattr(sys, 'argv', ['test.py'])

    continuous_integration()
    assert not CIRunner.run.called

def test_continuous_integration__set_browser(monkeypatch, mockfs_with_config, mock_CI_run):
    monkeypatch.setattr(sys, 'argv', ["test.py", "--browser", "firefox"])

    continuous_integration()

    CIRunner.run.assert_called_once_with(browser='firefox', logs=False)


def test_continuous_integration__show_logs(monkeypatch, mockfs_with_config, mock_CI_run):
    monkeypatch.setattr(sys, 'argv', ["test.py", "--logs"])

    continuous_integration()

    CIRunner.run.assert_called_once_with(logs=True, browser=None)


def test_continuous_integration__browser_default(monkeypatch, mockfs_with_config, mock_CI_run):
    monkeypatch.setattr(sys, 'argv', ["test.py"])

    continuous_integration()

    CIRunner.run.assert_called_once_with(browser=None, logs=False)


@pytest.fixture
def mock_App_run(request):
    App.run = MagicMock(name='run')


def test_standalone__set_host(monkeypatch, mock_App_run, mockfs_with_config):
    monkeypatch.setattr(sys, 'argv', ["test.py", "-o", "127.0.0.2"])
    App.run = MagicMock(name='run')

    standalone()

    App.run.assert_called_once_with(host="127.0.0.2", port=8888, debug=True)


def test_standalone__set_port(monkeypatch, mock_App_run, mockfs_with_config):
    monkeypatch.setattr(sys, 'argv', ["test.py", "-p", "1337"])

    standalone()

    App.run.assert_called_once_with(host="127.0.0.1", port=1337, debug=True)


def test_standalone__port_default(monkeypatch, mock_App_run, mockfs_with_config):
    monkeypatch.setattr(sys, 'argv', ["test.py"])

    standalone()

    App.run.assert_called_once_with(host="127.0.0.1", port=8888, debug=True)


def test_standalone__port_invalid(monkeypatch, mock_App_run, mockfs_with_config):
    monkeypatch.setattr(sys, 'argv', ["test.py", "-p", "pants"])

    with pytest.raises(SystemExit) as exception:
       standalone()

    assert "invalid int value: 'pants'"
    assert not App.run.called


def test_standalone__missing_config(monkeypatch, mock_App_run, mockfs):
    monkeypatch.setattr(sys, 'argv', ["test.py"])

    standalone()

    assert not App.run.called


def test__query__yes(capsys, monkeypatch):
    input_string(monkeypatch, "Y")

    choice = _query("Would you like to play a game?")

    out, err = capsys.readouterr()

    assert out == "Would you like to play a game? [Y/n] "
    assert True == choice


def test__query__no(capsys, monkeypatch):
    input_string(monkeypatch, "N")

    choice = _query("Would you like to hear some cat facts?")

    out, err = capsys.readouterr()

    assert out == "Would you like to hear some cat facts? [Y/n] "
    assert False == choice


def test__query__default(capsys, monkeypatch):
    input_string(monkeypatch, "")

    choice = _query("Would you like to blindly accept my defaults?")

    out, err = capsys.readouterr()

    assert out == "Would you like to blindly accept my defaults? [Y/n] "
    assert True == choice


def test__mkdir_p(mockfs):
    mkdir_p("/pants/a/b/c")

    assert os.path.isdir("/pants")
    assert os.path.isdir("/pants/a")
    assert os.path.isdir("/pants/a/b")
    assert os.path.isdir("/pants/a/b/c")


def test__mkdir_p(mockfs, monkeypatch):
    def raise_error(path):
        o = OSError()
        o.errno = errno.EEXIST
        raise o

    monkeypatch.setattr(os, 'makedirs', raise_error)
    mockfs.add_entries({
        '/pants/a/b/c': {}
    })

    mkdir_p("/pants/a/b/c")


def input_string(monkeypatch, string=""):
    try:
        monkeypatch.setattr(builtins, 'raw_input', lambda: string)
    except AttributeError:
        monkeypatch.setattr(builtins, 'input', lambda: string)


def test_install__yes(mockfs, monkeypatch):
    # Should create spec/javascripts/support
    # Should create a spec/javascripts/support/jasmine.yml
    spec_dir = "spec/javascripts/support"
    yaml_file = os.path.join(spec_dir, "jasmine.yml")

    input_string(monkeypatch, "Y")

    install()

    assert os.path.isdir(spec_dir)
    assert os.path.isfile(yaml_file)

    yaml = load(open(yaml_file))

    assert yaml['spec_files'] == ["**/*[Ss]pec.js"]


def test_install__yes__existing_yaml(mockfs, monkeypatch):
    # Should create spec/javascripts/support
    # Should NOT overwrite spec/javascripts/support/jasmine.yml
    spec_dir = "spec/javascripts/support"
    yaml_file = os.path.join(spec_dir, "jasmine.yml")

    mockfs.add_entries({
        '/spec/javascripts/support/jasmine.yml': """
        spec_files:
            - "**/pants.*"
        """
    })

    input_string(monkeypatch, "Y")

    install()

    assert os.path.isdir(spec_dir)
    assert os.path.isfile(yaml_file)

    yaml = load(open(yaml_file))

    assert yaml['spec_files'] == ["**/pants.*"]


def test_install__no(mockfs, monkeypatch):
    # Should NOT create spec/javascripts/support
    # Should NOT create a spec/javascripts/support/jasmine.yml
    spec_dir = "spec/javascripts/support"
    yaml_file = os.path.join(spec_dir, "jasmine.yml")

    input_string(monkeypatch, "N")

    install()

    assert not os.path.isdir(spec_dir)
    assert not os.path.isfile(yaml_file)



import datetime
import time
import sys
try:
    from io import StringIO
except ImportError:
    from StringIO import StringIO

from mock import MagicMock, Mock
import pytest

from jasmine.ci import CIRunner, TestServerThread

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict


def test_possible_ports():
    ports = TestServerThread().possible_ports("localhost:80,8000-8002")
    assert ports == [80, 8000, 8001, 8002]

@pytest.fixture
def sysexit(monkeypatch):
    mock_exit = MagicMock()
    monkeypatch.setattr(sys, 'exit', mock_exit)
    return mock_exit

@pytest.fixture
def test_server(monkeypatch):
    import jasmine.ci
    server = MagicMock()
    server.port = 80
    monkeypatch.setattr(jasmine.ci, 'TestServerThread', server)
    return server

@pytest.fixture
def firefox_driver(monkeypatch):
    import selenium.webdriver.firefox.webdriver
    driver = MagicMock()
    driver_class = lambda: driver
    monkeypatch.setattr(selenium.webdriver.firefox.webdriver, 'WebDriver', driver_class)
    return driver

@pytest.fixture
def suites():
    return [
        {
            "id": 0,
            "name": "datepicker",
            "type": "suite",
            "children": [
                {
                    "id": 0,
                    "name": "calls the datepicker constructor",
                    "type": "spec",
                    "children": []
                },
                {
                    "id": 1,
                    "name": "icon triggers the datepicker",
                    "type": "spec",
                    "children": []
                }
            ]
        }
    ]


@pytest.fixture
def results():
    return [
        {
            "id": "spec0",
            "description": "",
            "fullName": "",
            "status": "failed",
            'failedExpectations': [
                {
                    "matcherName": "toHaveBeenCalledWith",
                    "expected": {
                        "format": "yyy-mm-dd"
                    },
                    "actual": {},
                    "message": "Expected spy datepicker to have been called with [ { format : 'yyy-mm-dd' } ] but actual calls were [ { format : 'yyyy-mm-dd' } ]",
                    "stack": "Totally the one you want",
                    "passed": False
                }

            ]
        },
        {
            "id": "spec1",
            "description": "",
            "fullName": "",
            "status": "passed",
            "failedExpectations": []
        }
    ]

@pytest.fixture
def suite_results():
    return [
            {
                "id": "suite0",
                "status": "failed",
                "failedExpectations": [
                    { "message": "something went wrong"}
                 ]
            },
            {
                "id": "suite1",
                "status": "finished"
            }
           ]

def test_run_exits_with_zero_on_success(suites, results, capsys, sysexit, firefox_driver, test_server):
    results[0] = results[1]
    del results[1]

    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return []
        return None

    def get_log(type):
        return [dict(timestamp=0, level='INFO', message='hello')]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner().run(logs=True)
    stdout, _stderr = capsys.readouterr()

    assert not sysexit.called
    stdout, _stderr = capsys.readouterr()

    dt = datetime.datetime.fromtimestamp(0)
    assert '[{0} - INFO] hello\n'.format(dt) not in stdout


def test_run_exits_with_nonzero_on_failure(suites, results, capsys, sysexit, firefox_driver, test_server):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return []
        return None

    timestamp = time.time() * 1000

    def get_log(type):
        assert type == 'browser'
        return [
            dict(timestamp=timestamp, level='INFO', message='hello'),
            dict(timestamp=timestamp + 1, level='WARNING', message='world'),
        ]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner().run()

    sysexit.assert_called_with(1)
    stdout, _stderr = capsys.readouterr()

    assert "Browser Session Logs" not in stdout
    assert "hello" not in stdout
    assert "world" not in stdout


def test_run_with_browser_logs(suites, results, capsys, sysexit, firefox_driver, test_server):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return []
        return None

    timestamp = time.time() * 1000

    def get_log(type):
        assert type == 'browser'
        return [
            dict(timestamp=timestamp, level='INFO', message='hello'),
            dict(timestamp=timestamp + 1, level='WARNING', message='world'),
        ]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner().run(logs=True)

    stdout, _stderr = capsys.readouterr()

    dt = datetime.datetime.fromtimestamp(timestamp / 1000.0)
    assert '[{0} - INFO] hello\n'.format(dt) in stdout

    dt = datetime.datetime.fromtimestamp((timestamp + 1) / 1000.0)
    assert '[{0} - WARNING] world\n'.format(dt) in stdout


def test_displays_afterall_errors(suite_results, suites, results, capsys, sysexit, firefox_driver, test_server):
    results[0] = results[1]
    del results[1]

    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        return None

    firefox_driver.execute_script = execute_script

    CIRunner().run()
    stdout, _stderr = capsys.readouterr()

    assert 'After All something went wrong' in stdout
    sysexit.assert_called_with(1)




# -*- coding: utf-8 -*-
from __future__ import print_function
from .standalone import app as App
from jasmine.ci import CIRunner
import os
import argparse

def standalone():
    parser = argparse.ArgumentParser(description='Jasmine Standalone')
    parser.add_argument('-p', '--port', type=int, default=8888,
            help='The port of the Jasmine html runner')
    parser.add_argument('-o', '--host', type=str, default='127.0.0.1',
            help='The host of the Jasmine html runner')
    args = parser.parse_args()

    if _check_for_config():
        try:
             App.run(host=args.host, port=args.port, debug=True)
        except socket.error:
            sys.stdout.write('Socket unavailable')


def continuous_integration():
    parser = argparse.ArgumentParser(description='Jasmine-CI')
    parser.add_argument('-b', '--browser', type=str,
                        help='The selenium driver to utilize')
    parser.add_argument('-l', '--logs', action='store_true',
                        help='Displays browser logs')
    args = parser.parse_args()

    if _check_for_config(): CIRunner().run(browser=args.browser, logs=args.logs)

def _check_for_config():
    project_path = os.path.realpath(os.path.dirname(__name__))
    config_file = os.path.join(project_path, "spec/javascripts/support/jasmine.yml")

    config_exists = os.path.exists(config_file)
    if not config_exists:
        print("Could not find your config file at {0}".format(config_file))
    return config_exists


def _query(question):
    import sys

    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    prompt = " [Y/n] "

    while True:
        sys.stdout.write(question + prompt)

        try:
            choice = raw_input().lower()
        except NameError:
            choice = input().lower()

        if choice == '':
            return True
        elif choice in valid:
            return valid[choice]


def install():
    import os
    from jasmine.console import Formatter

    spec_dir = os.path.join(os.getcwd(), 'spec/javascripts/')

    print(Formatter.JASMINE_HEADER)

    print('Spec directory')

    if _query("About to create {0}... is this okay?".format(spec_dir)):
        print('making spec/javascripts')
        mkdir_p(spec_dir)

    yaml_dir = os.path.join(spec_dir, 'support')
    yaml_file_path = os.path.join(yaml_dir, 'jasmine.yml')

    print(("*" * 80) + '\n\nConfig yaml')

    if os.path.exists(yaml_file_path):
        print('found existing {0}, not overwriting'.format(yaml_file_path))
    else:
        if _query("About to create {0}... is this okay?".format(yaml_file_path)):
            print('making {0}'.format(yaml_dir))

            mkdir_p(yaml_dir)

            print('making {0}'.format(yaml_file_path))
            try:
                with open(yaml_file_path, 'w') as f:
                    f.write(YAML_TEMPLATE)
                    f.flush()
            except IOError:
                pass


def mkdir_p(path):
    import os
    import errno

    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


YAML_TEMPLATE = """
# src_files
#
# Return an array of filepaths relative to src_dir to include before jasmine specs.
# Default: []
#
# EXAMPLE:
#
# src_files:
#   - lib/source1.js
#   - lib/source2.js
#   - dist/**/*.js
#
src_files:

# stylesheets
#
# Return an array of stylesheet filepaths relative to src_dir to include before jasmine specs.
# Default: []
#
# EXAMPLE:
#
# stylesheets:
#   - css/style.css
#   - stylesheets/*.css
#
stylesheets:

# helpers
#
# Return an array of filepaths relative to spec_dir to include before jasmine specs.
# Default: ["helpers/**/*.js"]
#
# EXAMPLE:
#
# helpers:
#   - helpers/**/*.js
#
helpers:
  - "helpers/**/*.js"

# spec_files
#
# Return an array of filepaths relative to spec_dir to include.
# Default: ["**/*[sS]pec.js"]
#
# EXAMPLE:
#
# spec_files:
#   - **/*[sS]pec.js
#
spec_files:
  - "**/*[Ss]pec.js"


# src_dir
#
# Source directory path. Your src_files must be returned relative to this path. Will use root if left blank.
# Default: project root
#
# EXAMPLE:
#
# src_dir: public
#
src_dir:

# spec_dir
#
# Spec directory path. Your spec_files must be returned relative to this path.
# Default: spec/javascripts
#
# EXAMPLE:
#
# spec_dir: spec/javascripts
#
spec_dir: spec/javascripts
"""



import os
import threading
import sys

from selenium.common.exceptions import WebDriverException
from selenium.webdriver.support.wait import WebDriverWait

from cherrypy import wsgiserver

from jasmine.standalone import app as App
from jasmine.console import Parser, Formatter

import socket

class TestServerThread(threading.Thread):
    def run(self):
        ports = self.possible_ports("localhost:80,8889-9999")

        for index, port in enumerate(ports):
            try:
                self.server = wsgiserver.CherryPyWSGIServer(('localhost', port), App, request_queue_size=2048)
                self.port = port
                self.server.start()
                break
            except socket.error:
                continue

    def join(self, timeout=None):
        self.server.stop()

    def possible_ports(self, specified_address):
        possible_ports = []

        try:
            host, port_ranges = specified_address.split(':')
            for port_range in port_ranges.split(','):
                # A port range can be of either form: '8000' or '8000-8010'.
                extremes = list(map(int, port_range.split('-')))
                assert len(extremes) in [1, 2]
                if len(extremes) == 1:
                    # Port range of the form '8000'
                    possible_ports.append(extremes[0])
                else:
                    # Port range of the form '8000-8010'
                    for port in range(extremes[0], extremes[1] + 1):
                        possible_ports.append(port)
        except Exception:
            raise 'Invalid address ("{}") for live server.'.format(specified_address)

        return possible_ports


class CIRunner(object):
    def run(self, browser=None, logs=False):
        try:
            test_server = TestServerThread()
            test_server.start()

            driver = browser if browser else os.environ.get('JASMINE_BROWSER', 'firefox')

            try:
                webdriver = __import__("selenium.webdriver.{0}.webdriver".format(driver), globals(), locals(), ['object'], 0)

                self.browser = webdriver.WebDriver()
            except ImportError as e:
                print("Browser {0} not found".format(driver))

            self.browser.get("http://localhost:{0}/".format(test_server.port))

            WebDriverWait(self.browser, 100).until(
                lambda driver: driver.execute_script("return jsApiReporter.finished;")
            )

            spec_results = []
            index = 0
            batch_size = 50
            parser = Parser()

            while True:
                results = self.browser.execute_script("return jsApiReporter.specResults({0}, {1})".format(index, batch_size))
                results = parser.parse(results)

                spec_results.extend(results)
                index += len(results)

                if not len(results) == batch_size:
                    break

            suite_results = []
            index = 0
            batch_size = 50
            while True:
                results = self.browser.execute_script("return jsApiReporter.suiteResults({0}, {1})".format(index, batch_size))
                results = parser.parse(results)

                suite_results.extend(results)
                index += len(results)

                if not len(results) == batch_size:
                    break

            log = []
            if logs:
                try:
                    log = self.browser.get_log('browser')
                except WebDriverException:
                    pass

            formatter = Formatter(spec_results, suite_results=suite_results, browser_logs=log)
            sys.stdout.write(formatter.format())
            if len(list(formatter.results.failed())) or len(list(formatter.suite_results.failed())):
                sys.exit(1)
        finally:
            if hasattr(self, 'browser'):
                self.browser.close()
            if hasattr(self, 'test_server'):
                self.test_server.join()



from django.conf import settings


def pytest_configure():
    settings.configure(
        DEBUG=True,
        TEMPLATE_DEBUG=True,
        TEMPLATE_DIRS=('/templates',),
        PROJECT_PATH="/",
        INSTALLED_APPS=('jasmine',),
        JASMINE_YAML='jasmine.yml'
    )


import re
from setuptools import setup, find_packages

requirements = []
non_pypi_requirements = []

with open("requirements.txt", 'r') as f:
    for requirement in f:
        requirement = requirement.strip()

        if requirement.startswith("#") or not requirement:
            continue
        elif requirement.startswith("http"):
            non_pypi_requirements.append(requirement)
            requirements.append(re.match(r'.*#egg=(?P<egg>.*)', requirement).group('egg'))
        else:
            requirements.append(requirement)

setup(
    name="jasmine",
    version="2.2.0",
    url="http://pivotal.github.io/jasmine/",
    author="Pivotal Labs",
    author_email="jasmine-js@googlegroups.com",
    description=('Jasmine is a Behavior Driven Development testing framework for JavaScript. It does not rely on '+
                 'browsers, DOM, or any JavaScript framework. Thus it\'s suited for websites, '+
                 'Node.js (http://nodejs.org) projects, or anywhere that JavaScript can run.'),
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ],

    packages=find_packages(),
    package_data={'jasmine.django.templates': ['*.html']},
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'jasmine = jasmine.entry_points:standalone',
            'jasmine-ci = jasmine.entry_points:continuous_integration',
            'jasmine-install = jasmine.entry_points:install'
        ]
    },

    install_requires=requirements,

    dependency_links=non_pypi_requirements
)



from django.views.generic.base import TemplateView


class JasmineRunner(TemplateView):
    config = None

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)

        self.config.reload()

        js_files = self.config.script_urls()

        css_files = self.config.stylesheet_urls()

        context.update({
            'css_files': css_files,
            'js_files': js_files,
        })

        return self.render_to_response(context)


import os

from django.conf import settings
from django.conf.urls import patterns, url
from jasmine_core import Core

from jasmine.config import Config
from jasmine.django.views import JasmineRunner


def _config_file():
    default_location = os.path.join(settings.PROJECT_PATH, 'spec/javascripts/support/jasmine.yml')

    return settings.JASMINE_YAML if hasattr(settings, 'JASMINE_YAML') else default_location

config = Config(_config_file(), project_path=settings.PROJECT_PATH)
favicon_path = os.path.dirname(Core.favicon_path())

urlpatterns = patterns(
    '',
    url(r'^__spec__/(?P<path>.*)$', 'django.views.static.serve', {'document_root': config.spec_dir(), }, name='jasmine_specs'),
    url(r'^__src__/(?P<path>.*)$', 'django.views.static.serve', {'document_root': config.src_dir(), }, name='jasmine_src'),
    url(r'^__boot__/(?P<path>.*)$', 'django.views.static.serve', {'document_root': Core.boot_dir(), }, name='jasmine_boot'),
    url(r'^__jasmine__/(?P<path>.*)$', 'django.views.static.serve', {'document_root': Core.path(), }, name='jasmine_core'),
    url(r'^(?P<path>.*\.png)$', 'django.views.static.serve', {'document_root': favicon_path},),
    url(r'^$', JasmineRunner.as_view(template_name="runner.html", config=config), name='jasmine_runner'),

)



import datetime
import pytest
from jasmine.console import Parser, Formatter


def test_parser_should_return_a_correct_results_list():
    parser = Parser()

    results = parser.parse([
        {u'status': u'failed',
         u'fullName': u'Globals refer to the most holy.',
         u'failedExpectations': [{u'actual': u'Batman',
                                  u'matcherName': u'toEqual',
                                  u'passed': False,
                                  u'expected': u'PANTS',
                                  u'message': u"Expected 'Batman' to equal 'PANTS'.",
                                  u'stack': u"stack\n    stack\n    stack"}],

         u'passedExpectations': [{u'matcherName': u'toBeTruthy',
                                  u'expected': [],
                                  u'actual': True,
                                  u'message': u'Passed.',
                                  u'stack': u'',
                                  u'passed': True}],
         u'id': 0,
         u'description': u'refer to the most holy',
         u'pendingReason': u'pending reason'}
    ])

    assert len(results) == 1
    assert results[0].status == 'failed'
    assert results[0].fullName == 'Globals refer to the most holy.'
    assert len(results[0].failedExpectations) == 1
    assert results[0].failedExpectations[0]['stack'] == "stack\n    stack\n    stack"
    assert results[0].failedExpectations[0]['message'] == "Expected 'Batman' to equal 'PANTS'."
    assert results[0].pendingReason == u'pending reason'


@pytest.fixture
def results():
    parser = Parser()
    return parser.parse([
        {u'status': u'passed'},
        {u'status': u'failed'},
        {u'status': u'passed'},
        {u'status': u'pending', u'fullName': u'Context is this test is pending'},
    ])

@pytest.fixture
def passing_results():
    parser = Parser()
    return parser.parse([
        {u'status': u'passed'},
        {u'status': u'passed'},
        {u'status': u'pending', u'fullName': u'Context is this test is pending'},
    ])

@pytest.fixture
def browser_logs():
    return [
        {u'timestamp': 0, 'level': 'INFO', 'message': 'hi'},
        {u'timestamp': 1000, 'level': 'WARNING', 'message': 'lo'},
        {u'timestamp': 2000, 'level': 'INFO', 'message': 'bye'},
    ]


def test_format_progress(results):
    formatter = Formatter(results, colors=False)

    assert formatter.format_progress() == ".X.*"


def test_format_summary(results):
    formatter = Formatter(results, colors=False)

    assert formatter.format_summary() == "4 specs, 1 failed, 1 pending"

def test_format_after_all_errors():
    parser = Parser()
    suite_results = parser.parse([{u'status': u'failed', u'failedExpectations': [{"message": "ahhh"}]}])

    formatter = Formatter([], suite_results=suite_results)
    assert "After All ahhh" in formatter.format_suite_failure()

def test_format_browser_logs(results, browser_logs):
    formatter = Formatter(results, colors=False, browser_logs=browser_logs)

    dt1, dt2, dt3 = map(datetime.datetime.fromtimestamp, range(3))
    assert formatter.format_browser_logs() == (
        "Browser Session Logs:\n" +
        "  [{0} - INFO] hi\n".format(dt1) +
        "  [{0} - WARNING] lo\n".format(dt2) +
        "  [{0} - INFO] bye\n".format(dt3) +
        "\n"
    )

def test_format_browser_logs_with_no_failures(passing_results, browser_logs):
    formatter = Formatter(passing_results, colors=False, browser_logs=browser_logs)

    assert formatter.format_browser_logs() == ""

def test_format_failures():
    parser = Parser()

    stack1 = u"""Error: Expected 'Batman' to equal 'PANTS'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:3"""

    stack2 = u"""Error: Expected 'Batman' to equal 'Superman'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:6"""

    results = parser.parse([
        {u'status': u'passed', u'fullName': u'Context is this test passes'},
        {u'status': u'failed', u'fullName': u'Context is this test fails',
            u'failedExpectations': [{u'stack': stack1, u'message': 'Message1'}]},
        {u'status': u'failed', u'fullName': u'Context is this test also fails',
            u'failedExpectations': [{u'stack': stack2, u'message': 'Message2'}]},
    ])

    formatter = Formatter(results, colors=False)

    assert formatter.format_failures() ==\
        "Context is this test fails\n" +\
        "  Message1\n" +\
        "  Error: Expected 'Batman' to equal 'PANTS'.\n" +\
        "        at http://localhost:8888/__spec__/global_spec.js:3\n" +\
        "Context is this test also fails\n" +\
        "  Message2\n" +\
        "  Error: Expected 'Batman' to equal 'Superman'.\n" +\
        "        at http://localhost:8888/__spec__/global_spec.js:6\n"


def test_clean_stack(results):
    formatter = Formatter(results, colors=False)

    dirty_stack = u"""Error: Expected 'Batman' to equal 'PANTS'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:3"""

    assert formatter.clean_stack(dirty_stack) == """Error: Expected 'Batman' to equal 'PANTS'.
        at http://localhost:8888/__spec__/global_spec.js:3"""


def test_pending_stack(results):
    formatter = Formatter(results, colors=False)

    assert formatter.format_pending() == "Context is this test is pending\n"

def test_pending_with_message():
    parser = Parser();

    results = parser.parse([{
        u'status': u'pending',
        u'fullName': u'pending',
        u'pendingReason': 'the reason'
    }]);

    formatter = Formatter(results, colors=False)
    assert formatter.format_pending() == "pending\n  Reason: the reason\n"




import datetime
from collections import namedtuple
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

try:
    from future_builtins import filter
except ImportError:
    pass


class Formatter(object):
    COLORS = {
        'red': "\033[0;31m",
        'green': "\033[0;32m",
        'yellow': "\033[0;33m",
        'none': "\033[0m"
    }

    JASMINE_HEADER = """
      _                     _
     | |                   (_)
     | | __ _ ___ _ __ ___  _ _ __   ___
 _   | |/ _` / __| '_ ` _ \| | '_ \ / _ \\
| |__| | (_| \__ \ | | | | | | | | |  __/
 \____/ \__,_|___/_| |_| |_|_|_| |_|\___|

"""

    def __init__(self, results, **kwargs):
        self.colors = kwargs.get('colors', True)
        self.browser_logs = kwargs.get('browser_logs', [])
        self.suite_results = ResultList(kwargs.get('suite_results', []))
        self.results = ResultList(results)

    def colorize(self, color, text):
        if not self.colors:
            return text

        return self.COLORS[color] + text + self.COLORS['none']

    def format(self):
        return (
            self.JASMINE_HEADER +
            self.format_progress() + "\n\n" +
            self.format_summary() + "\n\n" +
            self.format_suite_failure() + "\n\n" +
            self.format_browser_logs() +
            self.format_failures() +
            self.format_pending()
        )

    def format_progress(self):
        output = ""

        for result in self.results:
            if result.status == "passed":
                output += self.colorize('green', '.')
            elif result.status == "failed":
                output += self.colorize('red', 'X')
            else:
                output += self.colorize('yellow', '*')

        return output

    def format_summary(self):
        output = "{0} specs, {1} failed".format(len(self.results), len(list(self.results.failed())))

        if self.results.pending():
            output += ", {0} pending".format(len(list(self.results.pending())))

        return output

    def format_suite_failure(self, colors=False):
        output = ""
        for result in self.suite_results:
            if(result.failedExpectations):
                output += self.colorize('red', '\nAfter All {0}'.format(result.failedExpectations[0]['message']))
        return output

    def format_browser_logs(self):
        output = ""
        if list(self.results.failed()) and self.browser_logs:
            output = "Browser Session Logs:\n"
            for log in self.browser_logs:
                output += "  [{0} - {1}] {2}\n".format(
                    datetime.datetime.fromtimestamp(log['timestamp'] / 1000.0),
                    log['level'],
                    log['message']
                )
            output += "\n"
        return output

    def format_failures(self):
        output = ""
        for failure in self.results.failed():
            output += self.colorize('red', failure.fullName) + "\n  " +\
                    failure.failedExpectations[0]['message'] + "\n  " +\
                    self.clean_stack(failure.failedExpectations[0]['stack']) + "\n"

        return output

    def clean_stack(self, stack):
        if not stack:
            return ""

        def dirty(stack_line):
            return "__jasmine__" in stack_line or "__boot__" in stack_line

        return "\n".join([stack_line for stack_line in stack.split("\n") if not dirty(stack_line)])

    def format_pending(self):
        output = ""

        for pending in self.results.pending():
            output += self.colorize('yellow', pending.fullName + "\n")
            if pending.pendingReason:
                output += "  Reason: {0}\n".format(pending.pendingReason)

        return output


class ResultList(list):
    def passed(self):
        return self._filter_status('passed')

    def failed(self):
        return self._filter_status('failed')

    def pending(self):
        return self._filter_status('pending')

    def _filter_status(self, status):
        return filter(lambda x: x.status == status, self)


RESULT_FIELDS = ['status', 'fullName', 'failedExpectations', 'id', 'description', 'pendingReason']

class Parser(object):
    def parse(self, items):
        return ResultList([Result(**item) for item in self._filter_fields(items)])

    def _filter_fields(self, raw_items):
        filtered_items = []
        for item in raw_items:
            filtered_items.append(dict(((k,v) for k,v in item.items() if k in RESULT_FIELDS)))
        return filtered_items


class Result(namedtuple('Result', RESULT_FIELDS)):
    def __new__(cls, status=None, fullName=None, failedExpectations=[], id=None, description=None, pendingReason=None):
        # Constructor arguments correspond to RESULT_FIELDS
        return super(Result, cls).__new__(cls, status, fullName, failedExpectations, id, description, pendingReason)

